#version 330 core

//Interpolierte Werte vom Vertexshader uebergeben
in vec2 fUVs;
in vec3 position_wspc;
in vec3 normal_cspc;
in vec3 viewdir_cspc;
in vec4 tangent_cspc;
//in vec3 lightdir_cspc;
//in mat3 TBN;

//Output (Farbe des Pixels)
out vec4 outcolor;

struct Light
{
	vec3 position;
	vec3 color;
	float power;
	vec2 near_far;
	int unit;
	float Padding;
};

layout (std140) uniform Lights
{
	int numberOfLights;
	Light light[4];
};

layout(std140)uniform Material {
	vec4 diffuse;
	vec4 ambient;
	vec4 specular;
	vec4 emissive;
	float shininess;	
	float UBOPadding1;
	float UBOPadding2;
	float UBOPadding3;
};

//Konstante Werte frs ganze Mesh
uniform sampler2D colorTexture;
uniform sampler2D normal_map;
uniform samplerCube depth_map;
//uniform vec3 lPosition_wspc;
uniform mat4 viewMatrix;
//if 1 true, if 0 false
uniform int normalMapping;
uniform int shadowMapping;
uniform float texCount;
//uniform material Material;

vec3 gridSamplingDisk[20] = vec3[]
(
   vec3(1, 1, 1), vec3(1, -1, 1), vec3(-1, -1, 1), vec3(-1, 1, 1), 
   vec3(1, 1, -1), vec3(1, -1, -1), vec3(-1, -1, -1), vec3(-1, 1, -1),
   vec3(1, 1, 0), vec3(1, -1, 0), vec3(-1, -1, 0), vec3(-1, 1, 0),
   vec3(1, 0, 1), vec3(-1, 0, 1), vec3(1, 0, -1), vec3(-1, 0, -1),
   vec3(0, 1, 1), vec3(0, -1, 1), vec3(0, -1, -1), vec3(0, 1, -1)
);

float evaluate_shadow(int i) {
	float result=1.0;
	float epsilon=0.15;	
	int numOfSamples=4;

	for(int a=0;a<numOfSamples;a++){
		int index = a;
		vec3 cm_lookup_vec= position_wspc.xyz - light[i].position;
		float curr_fragment_dist_to_light= length(cm_lookup_vec);
		float diskRadius = (1.0 + (1.0 - curr_fragment_dist_to_light) * 3) / 512; // /SizeofCubeTex
		
		float smallest_dist_to_light=1.0;
		vec3 baseDirection= cm_lookup_vec;
		vec3 baseOffset= gridSamplingDisk[a] * diskRadius;

		smallest_dist_to_light= texture(depth_map, baseDirection+baseOffset).r;
		
		smallest_dist_to_light *=light[i].near_far.y;

		if(smallest_dist_to_light+epsilon< curr_fragment_dist_to_light){
		result-=0.9/numOfSamples;
		}
	}

	return result;

}

vec3 colorcount;
void main(){

	vec3 mDiffuseColor;
	vec3 mAmbientColor;
	vec3 mSpecularColor;

	// normal map
	vec3 NewNormal;
	NewNormal = normal_cspc;

	if(normalMapping == 1 && texCount ==2){
	vec3 Normal = normal_cspc;
	vec3 Tangent = tangent_cspc.xyz;
	Tangent = normalize(Tangent - dot(Tangent, Normal) * Normal);
	vec3 Bitangent = cross(Normal,Tangent);
	Bitangent = normalize(Bitangent * tangent_cspc.w);
	vec3 normal_tan = texture (normal_map, fUVs).rgb;
	normal_tan = normalize (normal_tan * 2.0 - 1.0);
	 
	 mat3 TBN = mat3(Tangent, Bitangent, Normal);
	 NewNormal = TBN*normal_tan;
	 NewNormal = normalize(NewNormal);
	}

	// normal map samplen und von 0:1 auf -1:1 konvertieren

	float shadow_factor = 1.0;

	//Materialwerte 
	if(texCount == 0.0){
		mDiffuseColor= diffuse.rgb;
		mAmbientColor= ambient.rgb;
		mSpecularColor= specular.rgb;
	} else {
		mDiffuseColor= texture(colorTexture, fUVs).rgb;
		mAmbientColor= ambient.rgb * mDiffuseColor;
		mSpecularColor= specular.rgb;
	}

	vec3 color = vec3(0.0, 0.0, 0.0);
	
	for(int i=0; i<numberOfLights; i++){
	
		//Lichtwerte werden ausgelesen
		vec3 lColor= light[i].color;
		float lPower= light[i].power;

		vec3 lPosition_cspc= (viewMatrix * vec4(light[i].position,1)).xyz;
		vec3 lightdir_cspc= lPosition_cspc + viewdir_cspc;


		//Distanz zum Licht
		float distance= length(light[i].position - position_wspc);

		//Normale des zu berechnenden Fragments(N), im Cameraspace
		vec3 n = normalize(NewNormal);

		//Richtung des Lichtes(L)
		vec3 l= normalize(lightdir_cspc);

		//Cosinus des Winkels zwischen der Normale und der Lichtrichtung(cosNL)
		//Falls der Winkel negativ ist wird er auf 0,1 gesetzt 
		float cosNL= clamp( dot(l,n), 0,1);


		vec3 E= normalize(viewdir_cspc);
		vec3 R= reflect(-l,n);
		float cosAlpha= clamp(dot(E,R), 0,1);

		if(shadowMapping == 1 && light[i].unit==2){
		shadow_factor = evaluate_shadow(i);
		color+=		
		//Diffuse: "Farbe" des Objektes
		 shadow_factor *(mDiffuseColor * lColor * lPower * cosNL/(distance*distance)) +
		//Specular: reflektierter Glanzpunkt
		 shadow_factor *(mSpecularColor * lColor * lPower * pow(cosAlpha,5.0)/(distance*distance));
		} else {
		color+=		
		//Diffuse: "Farbe" des Objektes
		mDiffuseColor * lColor * lPower * cosNL/(distance*distance) +
		//Specular: reflektierter Glanzpunkt
		mSpecularColor * lColor * lPower * pow(cosAlpha,5.0)/(distance*distance);
		}

	}

	color+= mAmbientColor;
	outcolor=vec4(color, texture2D(colorTexture, fUVs).aaaa);
}