#version 330
#extension GL_ARB_shading_language_420pack : require

// Textures
layout (binding = 0) uniform sampler2D textureRefraction;
layout (binding = 1) uniform sampler2D textureDuDv;

uniform float time;

// Output
out vec4 fragColor;

in FragmentData
{
	vec4 position;	// Clip space
	vec2 texcoord;

} DataIn;

void main()
{	
	// Texture coordinates for dUdV map
	vec2 t = vec2(DataIn.texcoord.x, DataIn.texcoord.y + time);

	// Sample distortion
	vec2 dist = texture2D(textureDuDv, t).rg * 2 - 1;

	// Multiply distortion with alpha mask value
	vec2 texcoord = (DataIn.position.xy / DataIn.position.w) / 2 + 0.5;
	dist *= (1 - texture2D(textureRefraction, texcoord).a);
	vec2 texcoordRefraction = clamp(texcoord + dist * 0.0075, 0, 1);
	
	// Compute refraction
	vec4 refrA = texture2D(textureRefraction, texcoord);
	vec4 refrB = texture2D(textureRefraction, texcoordRefraction);
	refrB.a = 1 - float(refrB.a < 1);
	
	vec4 colorRefr = refrA * refrB.a + refrB * (1 - refrB.a);

	// Final color
	fragColor = colorRefr;
	fragColor.a = 1;
}