#version 330 core

// Uniforms
layout (std140) uniform Matrices
{
	mat4 m_pvm;
	mat4 m_view;
	mat4 m_viewModel;
	mat4 m_projection;
	mat4 m_normal;
};

layout (std140) uniform Particle
{
	vec4 p_startColor;
	vec4 p_endColor;
	uvec2 p_texImages;
	float p_texAnimSpeed;
	float p_life;
};

// Input layout
layout (points) in;
layout (triangle_strip, max_vertices = 4) out;

// Input data
in VertexData
{
	vec4 color;
	vec2 scale;
	float rotation;
	uint subImage;

} DataIn[1];

// Output data
out FragmentData
{
	vec4 color;
	vec2 texcoord;

} DataOut;

// Corners
const vec2 corners[4] = vec2[4]( vec2(0, 1), vec2(0, 0), vec2(1, 1), vec2(1, 0) );

void main()
{
	// Calculate frame
	vec2 uvFrame = vec2(1) / p_texImages;
	vec2 frame = vec2(mod(DataIn[0].subImage, p_texImages.x), DataIn[0].subImage / p_texImages.x);
	frame.y = p_texImages.y - frame.y - 1;

	// Precalculation for rotation
	float a = cos(DataIn[0].rotation);
	float b = sin(DataIn[0].rotation);

    for (int i = 0; i < 4; i++)
    {
		vec2 c = DataIn[0].scale * (corners[i] - vec2(0.5));
		c = vec2(a * c.x - b * c.y, b * c.x + a * c.y);

        gl_Position = m_projection * (gl_in[0].gl_Position + vec4(c, 0, 0));            
        DataOut.texcoord = (corners[i] + frame) * uvFrame; 
		DataOut.color = DataIn[0].color;

        EmitVertex();
    }
}