#version 330
#extension GL_ARB_shading_language_420pack : require

// Constants
const vec3 specular = vec3(1, 1, 1);
const float shininess = 32;

// Textures
layout (binding = 0) uniform sampler2D textureReflection;
layout (binding = 1) uniform sampler2D textureRefraction;
layout (binding = 2) uniform sampler2D textureDuDv;
layout (binding = 3) uniform sampler2D textureNormal;

// Uniforms
struct Light
{
	vec3 position;
	vec3 color;
	float power;
	vec2 nearFar;
	float Padding0;
	float Padding1;
};

layout (std140) uniform Lights
{
	int numberOfLights;
	Light light[3];
};

uniform float time;

// Input
in Data
{
	vec4 position;	// Clip space
	vec2 texcoord;
	vec3 eye;		// Tangent space
	vec3 light[3];	// Tangent space

} DataIn;

out vec4 fragColor;

void main()
{	
	// Texture coordinates for normal and dUdV map
	vec2 t0 = vec2(DataIn.texcoord.x + time, DataIn.texcoord.y);
	vec2 t1 = vec2(-DataIn.texcoord.x + time, DataIn.texcoord.y + time);

	// Sample normals
	vec3 n0 = texture2D(textureNormal, t0).rgb;
	vec3 n1 = texture2D(textureNormal, t1).rgb;
	n0.rg = n0.rg * 2 - 1; 
	n1.rg = n1.rg * 2 - 1;

	// Compute specular
	vec4 spec = vec4(0);
	vec3 n = normalize((n0 + n1) / 2);
	vec3 e = normalize(DataIn.eye);
	vec3 l = normalize(DataIn.light[0]);

	float NdotL = max(dot(n, l), 0);

	if (NdotL > 0)
	{
		// Compute reflection
		vec3 r = reflect(l, n);  
        
		// Compute the specular term into spec
		float intSpec = max(dot(r, e), 0);
		spec += vec4(specular * pow(intSpec, shininess), 1);
	}

	// Sample distortions
	vec2 dist0 = texture2D(textureDuDv, t0).rg * 2 - 1;
	vec2 dist1 = texture2D(textureDuDv, t1).rg * 2 - 1;
	vec2 dist = (dist0 + dist1) / 2;

	// Compute refraction and reflection
	vec2 texcoord = (DataIn.position.xy / DataIn.position.w) / 2 + 0.5;
	vec2 texcoordReflection = clamp(texcoord + dist * 0.005, 0, 1);
	vec2 texcoordRefraction = clamp(texcoord + dist * 0.01, 0, 1);
	
	vec4 reflA = texture2D(textureReflection, texcoord);
	vec4 reflB = texture2D(textureReflection, texcoordReflection);
	vec4 colorRefl = reflA * reflB.a + reflB * (1 - reflB.a);

	vec4 refrA = texture2D(textureRefraction, texcoord);
	vec4 refrB = texture2D(textureRefraction, texcoordRefraction);
	vec4 colorRefr = refrA * refrB.a + refrB * (1 - refrB.a);

	// Final color
	fragColor = mix(colorRefl + spec, colorRefr, dot(n, e));
	fragColor.a = 1;
}